' Tile Set
Type TSTileSet
	Field id		' tileset number
	Field width, height	' width, height of each tile
	Field horizTileCount, vertTileCount	' number of tiles (vertical, horizontal)
	Field tileCount		' total number of tiles
	Field mapCount		' number of maps
	Field bitmapWidth, bitmapHeight	' width, height of the bitmap file
	Field maskR, maskG, maskB	' mask color
	Field tiles:TImage		' the tileset image
	Field maps:TSMap[]
	Field seqs:TSSequence[]	' sequences
	
	' Load -----------------------------------------
	' Load the image file into an animated image
	Method Load(file$)
		SetMaskColor maskR,maskG,maskB
		tiles = LoadAnimImage(file$, width, height, 0, tileCount, MASKEDIMAGE)
	End Method
	
	' DrawTile -------------------------------------
	' Draw one tile
	Method DrawTile(number, x, y)
		DrawImage tiles,x,y,number
	End Method
	
	' DrawMap --------------------------------------
	' Draw a map
	Method DrawMap(number, x, y)
		Local map:TSMap = maps[number]
		Local nx# = x * map.scrollX#
		Local ny# = y * map.scrollY#
		
		For Local iy = 0 Until map.height
			For Local ix = 0 Until map.width
				Local mTile = map.cells[ix,iy].tile
				
				If (mTile < 0)
					Local seq = (mTile+1)
					mTile = seqs[seq].tiles[seqs[seq].curFrame]
				EndIf
				
				If (mTile > 0)
					DrawTile(mTile-1,nx+(ix*width),ny+(iy*height))
				EndIf
			Next
		Next
	End Method
End Type

' Map
Type TSMap
	Field id		' id number
	Field width, height		' width, height
	Field scrollX#, scrollY#		' scrollX, scrollY
	Field cells:TSMapCell[,]
End Type

' Map Cell
Type TSMapCell
	Field tile		' number of the tile
	Field bounds:Byte		' bounds (for collisions)
	Field mapCode:Byte		' map code (for placing objects)
End Type

' Sequence
Type TSSequence
	Field id
	Field frames
	Field tiles[] ' [frame(0..frames)]
	Field curFrame
End Type

' LoadMap ---------------------------------------
' Load tilesets and maps
Function LoadTileSets:TSTileSet[](mapfile$)
	Local f:TStream = ReadFile(mapfile)
	
	If f = Null
		Notify "Unable to open file: ~n"+mapfile$
		End
	EndIf

	Local numTileSets = ReadLine(f).ToInt()
	Local ts:TSTileSet[numTileSets]
	
	Local ln$, arr$[10]
	
	' Read Tile Sets
	For Local i = 0 Until numTileSets
		ts[i] = New TSTileSet
		Local file$ = ReadLine(f)
		ln$ = ReadLine(f)
		arr = SliceString(ln$,",")
		ts[i].id = arr[0].ToInt()
		ts[i].width = arr[1].ToInt()
		ts[i].height = arr[2].ToInt()
		ts[i].horizTileCount = arr[3].ToInt()
		ts[i].vertTileCount = arr[4].ToInt()
		ts[i].tileCount = ts[i].horizTileCount * ts[i].vertTileCount
		ln = ReadLine(f)
		arr = SliceString(ln,",")
		ts[i].bitmapWidth = arr[0].ToInt()
		ts[i].bitmapHeight = arr[1].ToInt()
		ts[i].maskR = arr[2].ToInt()
		ts[i].maskG = arr[3].ToInt()
		ts[i].maskB = arr[4].ToInt()
		ts[i].Load(file$)
		
		Local numMaps = ReadLine(f).ToInt()
		ts[i].maps = New TSMap[numMaps]
		
		' Read Maps
		For Local j = 0 Until numMaps
			ts[i].maps[j] = New TSMap
			ln = ReadLine(f)
			arr = SliceString(ln,",")
			ts[i].maps[j].id = arr[0].ToInt()
			ts[i].maps[j].width = arr[1].ToInt()
			ts[i].maps[j].height = arr[2].ToInt()
			ts[i].maps[j].scrollX = arr[3].ToFloat()
			ts[i].maps[j].scrollY = arr[4].ToFloat()
			
			ts[i].maps[j].cells = New TSMapCell[ts[i].maps[j].width, ts[i].maps[j].height]
			
			' Map Data
			For Local kj = 0 Until ts[i].maps[j].height
				For Local ki = 0 Until ts[i].maps[j].width
					ts[i].maps[j].cells[ki, kj] = New TSMapCell
					ln = ReadLine(f)
					arr = SliceString(ln,",")
					ts[i].maps[j].cells[ki, kj].tile = arr[0].ToInt()
					ts[i].maps[j].cells[ki, kj].bounds = arr[1].ToInt()
					ts[i].maps[j].cells[ki, kj].mapCode = arr[2].ToInt()
				Next
			Next
		Next	
		
		Local numSeqs = ReadLine(f).ToInt()
		ts[i].seqs = New TSSequence[numSeqs]
		
		' Read Sequences
		For Local s = 0 Until numSeqs
			ts[i].seqs[s] = New TSSequence
			ts[i].seqs[s].curFrame = 0
			ts[i].seqs[s].id = ReadLine(f).ToInt()
			ts[i].seqs[s].frames = ReadLine(f).ToInt()
			ts[i].seqs[s].tiles = New Int[ts[i].seqs[s].frames]
			
			For Local fr = 0 Until ts[i].seqs[s].frames
				ts[i].seqs[s].tiles[fr] = ReadLine(f).ToInt()
			Next
		Next
	Next
	
	CloseFile f
	FlushMem
	
	Return ts
End Function

' SliceString -----------------------------------
' Slice a string into an array of strings
Function SliceString:String[](str$, sep$)
	Local off = 0, oldOff = 0
	Local n = 0
	
	Repeat
		off = str.Find(sep$,off)
		off :+ 1
		n :+ 1
	Until off = 0
	
	Local arr$[n]
	off = 0
	Local i = 0
	
	Repeat
		off = str.Find(sep$,oldOff)
		
		If off = -1
			off = str.length
		EndIf
		
		arr[i] = str[oldOff..off]
		off :+ 1
		i :+ 1
		oldOff = off
	Until off = str.length+1

	Return arr
End Function